require "import" 
import "android.widget.*" 
import "com.androlua.*" 

layout = { 
    GridView, 
    id = "grid", 
    numColumns = 1, 
    layout_width = "fill", 
    layout_height = "fill" 
}

items = {"English", "Hindi", "Marathi", "Gujarati", "Bengali", "Kashmiri"}
dlg = LuaDialog(this)
dlg.View = loadlayout(layout)
grid.adapter = SingleLineAdapter(service, String(items))
dlg.setTitle("Select a language for voice typing")
dlg.setButton("Close", nil)
dlg.show()

grid.onItemClick = function(l, v)
    dlg.dismiss()
    task(20, function()
        local languageCode = {
            ["English"] = "en-IN",
            ["Hindi"] = "hi-IN",
            ["Marathi"] = "mr-IN",
            ["Gujarati"] = "gu-IN",
            ["Bengali"] = "bn-IN",
            ["Kashmiri"] = "ks-IN"
        }

        local selectedLanguage = v.text
        local selectedCode = languageCode[selectedLanguage]

        if selectedCode then
            require "import"
            import "android.content.Context"
            import "android.content.Intent"
            import "java.util.Locale"
            import "android.speech.SpeechRecognizer"
            import "android.speech.RecognitionListener"
            import "android.speech.RecognizerIntent"

            function destroySpeechRecognizer(rec)
                if rec then
                    rec.destroy()
                    rec = nil
                end
                return true
            end

            function startSpeechRecognition()
                local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)
                local recognitionListener = RecognitionListener {
                    onResults = function(results)
                        local resultArray = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
                        if resultArray ~= nil and resultArray.size() > 0 then
                            local recognizedText = resultArray.get(0)
                            local emoji = "😊"  -- emoji to append
                            local editText = service.getEditText()
                            service.insertText(editText, recognizedText .. " " .. emoji)
                            service.speak(recognizedText)
                        end
                        destroySpeechRecognizer(speechRecognizer)
                    end,
                    onError = function()
                        destroySpeechRecognizer(speechRecognizer)
                        return false
                    end
                }

                local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
                recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
                recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, selectedCode)
                recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())
                speechRecognizer.startListening(recognizerIntent)
                speechRecognizer.setRecognitionListener(recognitionListener)

                return true
            end

            startSpeechRecognition()
        end
    end)
end